﻿using Syncfusion.Data;
using Syncfusion.UI.Xaml.Grid;
using Syncfusion.UI.Xaml.Grid.RowFilter;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;

namespace SfDataGridDemo
{
    /// <summary>
    /// Class represents the GridTextFilterComboBoxRendererExt for customizing the ComboBoxRenderer
    /// </summary>
    public class GridMultiSelectComboBoxRendererExt : GridFilterRowMultiSelectRenderer, INotifyPropertyChanged
    {
        private List<string> textComboBoxItems;

        public GridMultiSelectComboBoxRendererExt()
            : base()
        {
            SetTextComboBoxItemsList();
        }

        /// <summary>
        ///  Generate the Items for TextComboBox
        /// </summary>
        /// <returns></returns>
        public void SetTextComboBoxItemsList()
        {
            textComboBoxItems = new List<string>();
            textComboBoxItems.Add("SelectAllExpect ALFKI");
            textComboBoxItems.Add("ALFKI");
            textComboBoxItems.Add("ANATR");
            textComboBoxItems.Add("AROUT");
            textComboBoxItems.Add("BLAUS");
            textComboBoxItems.Add("BOLID");
            textComboBoxItems.Add("BOTTM");
            textComboBoxItems.Add("BERGS");
            textComboBoxItems.Add("BLONP");
            textComboBoxItems.Add("BONAP");
            textComboBoxItems.Add("ANTON");
            
        }
        /// </summary>
        /// <param name="dataColumn"></param>
        /// <param name="uiElement"></param>
        /// <param name="dataContext"></param>
        public override void OnInitializeDisplayElement(Syncfusion.UI.Xaml.Grid.DataColumnBase dataColumn, ContentControl uiElement, object dataContext)
        {
            base.OnInitializeDisplayElement(dataColumn, uiElement, dataContext);
            uiElement.Margin = new Thickness(5, 0, 0, 0);
        }

        /// <summary>
        /// InitializeEditBinding based on our item, set the SelectedItem and set the ItemSource.
        /// </summary>
        /// <param name="uiElement">Corresponding uiElement</param>
        /// <param name="dataColumn">Corresponding Column</param>
        protected override void InitializeEditBinding(Syncfusion.Windows.Tools.Controls.ComboBoxAdv uiElement, DataColumnBase dataColumn)
        {
            ObservableCollection<object> selItems = new ObservableCollection<object>();

            //Generate the items for FilterRow 
            uiElement.ItemsSource = textComboBoxItems;
          //  InitializeTextFilter(dataColumn, selItems);
            if (selItems.Count > 0)
                uiElement.SelectedItems = selItems;
            else if (uiElement.SelectedItems != null)
                uiElement.SelectedItems = null;
            uiElement.AllowMultiSelect = true;
            uiElement.AllowSelectAll = true;
            uiElement.EnableOKCancel = true;
            uiElement.IsEditable = false;
        }

        /// <summary>
        /// Check whether the column having a filterperdicates or not
        /// </summary>
        /// <param name="filterPredicate">FilterPredicates for a column</param>
        /// <param name="filterValue">FilterValue for a column</param>
        /// <returns></returns>
        private bool NeedToAdd(ObservableCollection<FilterPredicate> filterPredicate, string filterValue)
        {
            bool needToAdd = false;
            foreach (var item in filterPredicate)
            {
                if ((item as FilterPredicate).FilterValue.ToString() == filterValue)
                {
                    needToAdd = true;
                    break;
                }
            }
            return needToAdd;
        }

        /// <summary>
        /// Generate the FilterPredicates and apply the filter for a corresponding column
        /// </summary>
        /// <param name="filterValues">Corresponding FilterValue</param>
        /// <param name="totalItems">Corresponding FilterItems</param>
        public override void ProcessMultipleFilters(List<object> filterValues, List<object> totalItems)
        {
            var selectedItems = filterValues.Cast<string>().ToList();
            var total = totalItems.Cast<string>().ToList();
            if (selectedItems == null || total == null || filterValues == null)
                return;

            if (selectedItems.Count == total.Count)
            {
                this.ApplyFilters(null, string.Empty);
                this.IsValueChanged = false;
                return;
            }
            var filterPredicates = new List<FilterPredicate>();
            if (filterValues.Count > 0)
            {
                if(selectedItems.Contains("SelectAllExpect ALFKI"))
                    ProcessTextFilter(filterPredicates, "SelectAllExpect ALFKI");
                else
                {
                    selectedItems.ForEach(item =>
                    {
                        ProcessTextFilter(filterPredicates, item);
                    });
                }
                
            }
            string _filterText = string.Empty;
            //Creates the FilterText
            if (filterPredicates.Count > 0)
            {
                for (int i = 0; i < selectedItems.Count; i++)
                {
                    _filterText += selectedItems[i];
                    if (i != selectedItems.Count - 1)
                        _filterText += " - ";
                }
                   
            }
            if (filterPredicates != null)
                this.ApplyFilters(filterPredicates, _filterText);
            this.IsValueChanged = false;
        }


        /// <summary>
        /// Generate the FiletrPredicates for StringTyped
        /// </summary>
        /// <param name="value"></param>
        /// <param name="filterType"></param>
        /// <param name="predType"></param>
        /// <returns></returns>
        private FilterPredicate GetStringFilterPredicates(object value, FilterType filterType, PredicateType predType)
        {
            return new FilterPredicate()
            {
                FilterBehavior = FilterBehavior.StringTyped,
                FilterType = filterType,
                FilterValue = value,
                IsCaseSensitive = false,
                PredicateType = predType
            };
        }


        /// <summary>
        /// Initialize the TextFilter
        /// </summary>
        /// <param name="dataColumn">Corresponding column</param>
        /// <param name="SelectedItem">SelectedItem for ComboBox</param>
        public void InitializeTextFilter(DataColumnBase dataColumn, ObservableCollection<object> SelectedItem)
        {
            if (dataColumn.GridColumn.FilteredFrom == FilteredFrom.FilterRow && dataColumn.GridColumn.FilterPredicates.Count > 0)
            {
                if (textComboBoxItems != null)
                {
                    textComboBoxItems.ForEach(element =>
                    {
                        //Check if the filter is already applied or not, if applied means again add the filter
                        bool needToAdd = false;
                        switch (element)
                        {
                            case "SelectAllExpect ALFKI":
                                needToAdd = this.NeedToAdd(dataColumn.GridColumn.FilterPredicates, "SelectAllExpect ALFKI");
                                break;
                            case "ALFKI":
                                needToAdd = this.NeedToAdd(dataColumn.GridColumn.FilterPredicates, "ALFKI");
                                break;
                            case "ANATR":
                                needToAdd = this.NeedToAdd(dataColumn.GridColumn.FilterPredicates, "ANATR");
                                break;
                            case "AROUT":
                                needToAdd = this.NeedToAdd(dataColumn.GridColumn.FilterPredicates, "AROUT");
                                break;
                            case "BLAUS":
                                needToAdd = this.NeedToAdd(dataColumn.GridColumn.FilterPredicates, "BLAUS");
                                break;
                            case "BOLID":
                                needToAdd = this.NeedToAdd(dataColumn.GridColumn.FilterPredicates, "BOLID");
                                break;
                            case "BOTTM":
                                needToAdd = this.NeedToAdd(dataColumn.GridColumn.FilterPredicates, "BOTTM");
                                break;
                            case "BERGS":
                                needToAdd = this.NeedToAdd(dataColumn.GridColumn.FilterPredicates, "BERGS");
                                break;
                            case "BLONP":
                                needToAdd = this.NeedToAdd(dataColumn.GridColumn.FilterPredicates, "BLONP");
                                break;
                            case "BONAP":
                                needToAdd = this.NeedToAdd(dataColumn.GridColumn.FilterPredicates, "BONAP");
                                break;
                            case "ANTON":
                                needToAdd = this.NeedToAdd(dataColumn.GridColumn.FilterPredicates, "ANTON");
                                break;

                        }
                        if (needToAdd)
                            SelectedItem.Add(element);
                    });
                }
            }
        }


        /// <summary>
        /// Created the filterpredicates for TextColumn
        /// </summary>
        /// <param name="filterPredicates"></param>
        /// <param name="item"></param>
        public void ProcessTextFilter(List<FilterPredicate> filterPredicates, string item)
        {
            switch (item)
            {
                case "SelectAllExpect ALFKI":
                    filterPredicates.Add(GetStringFilterPredicates("ANATR", FilterType.Equals, PredicateType.OrElse));
                    filterPredicates.Add(GetStringFilterPredicates("AROUT", FilterType.Equals, PredicateType.Or));
                    filterPredicates.Add(GetStringFilterPredicates("BLAUS", FilterType.Equals, PredicateType.Or));
                    filterPredicates.Add(GetStringFilterPredicates("BOLID", FilterType.Equals, PredicateType.Or));
                    filterPredicates.Add(GetStringFilterPredicates("BOTTM", FilterType.Equals, PredicateType.Or));
                    filterPredicates.Add(GetStringFilterPredicates("BERGS", FilterType.Equals, PredicateType.Or));
                    filterPredicates.Add(GetStringFilterPredicates("BLONP", FilterType.Equals, PredicateType.Or));
                    filterPredicates.Add(GetStringFilterPredicates("BONAP", FilterType.Equals, PredicateType.Or));
                    filterPredicates.Add(GetStringFilterPredicates("ANTON", FilterType.Equals, PredicateType.Or));
                    break;
                case "ALFKI":
                    filterPredicates.Add(GetStringFilterPredicates("ALFKI", FilterType.Equals, PredicateType.Or));
                    
                    break;
                case "ANATR":
                    filterPredicates.Add(GetStringFilterPredicates("ANATR", FilterType.Equals, PredicateType.Or));
                  
                    break;
                case "AROUT":
                    filterPredicates.Add(GetStringFilterPredicates("AROUT", FilterType.Equals, PredicateType.Or));
                   
                    break;
                case "BLAUS":
                    filterPredicates.Add(GetStringFilterPredicates("BLAUS", FilterType.Equals, PredicateType.Or));
                    
                    break;
                case "BOLID":
                    filterPredicates.Add(GetStringFilterPredicates("BOLID", FilterType.Equals, PredicateType.Or));
                   
                    break;

                case "BOTTM":
                    filterPredicates.Add(GetStringFilterPredicates("BOTTM", FilterType.Equals, PredicateType.Or));

                    break;

                case "BERGS":
                    filterPredicates.Add(GetStringFilterPredicates("BERGS", FilterType.Equals, PredicateType.Or));

                    break;

                case "BLONP":
                    filterPredicates.Add(GetStringFilterPredicates("BLONP", FilterType.Equals, PredicateType.Or));

                    break;

                case "BONAP":
                    filterPredicates.Add(GetStringFilterPredicates("BONAP", FilterType.Equals, PredicateType.Or));

                    break;

                case "ANTON":
                    filterPredicates.Add(GetStringFilterPredicates("ANTON", FilterType.Equals, PredicateType.Or));

                    break;
            }
        }


        public event PropertyChangedEventHandler PropertyChanged;
        private void OnPropertyChanged(String prop)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(prop));
            }
        }
    }
}
